@@ -500,130 +500,71 @@ s48_raise_closed_channel_error() {
 void
 s48_raise_os_error(int the_errno) {
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 2,
-			     s48_enter_integer(the_errno),
+			     s48_enter_fixnum(the_errno),
 			     s48_enter_string(strerror(the_errno)));
 }
 
 void
 s48_raise_os_error_1(int the_errno, s48_value arg1) {
-
-  s48_value sch_errno = S48_UNSPECIFIC;
-  
-  S48_DECLARE_GC_PROTECT(2);
-  S48_GC_PROTECT_2 (arg1, sch_errno);
-  
-  sch_errno = s48_enter_integer(the_errno);
-
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 3,
-			     sch_errno,
+			     s48_enter_fixnum(the_errno),
 			     s48_enter_string(strerror(the_errno)),
 			     arg1);
-
-  S48_GC_UNPROTECT();
 }
 
 void
 s48_raise_os_error_2(int the_errno, s48_value arg1, s48_value arg2) {
-
-  s48_value sch_errno = S48_UNSPECIFIC;
-  
-  S48_DECLARE_GC_PROTECT(3);
-  S48_GC_PROTECT_3 (arg1, arg2, sch_errno);
-  
-  sch_errno = s48_enter_integer(the_errno);
-
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 4,
-                             sch_errno,
+			     s48_enter_fixnum(the_errno),
 			     s48_enter_string(strerror(the_errno)),
 			     arg1, arg2);
-  S48_GC_UNPROTECT();
 }
 
 void
 s48_raise_os_error_3(int the_errno, s48_value arg1, s48_value arg2, 
 		     s48_value arg3) {
-
-  s48_value sch_errno = S48_UNSPECIFIC;
-  
-  S48_DECLARE_GC_PROTECT(4);
-  S48_GC_PROTECT_4 (arg1, arg2, arg3, sch_errno);
-  
-  sch_errno = s48_enter_integer(the_errno);
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 5,
-                             sch_errno,
+			     s48_enter_fixnum(the_errno),
 			     s48_enter_string(strerror(the_errno)),
 			     arg1, arg2, arg3);
-  S48_GC_UNPROTECT();
 }
 
 void
 s48_raise_os_error_4(int the_errno, s48_value arg1, s48_value arg2, 
 		     s48_value arg3, s48_value arg4) {
-
-  s48_value sch_errno = S48_UNSPECIFIC;
-  
-  S48_DECLARE_GC_PROTECT(5);
-  S48_GC_PROTECT_5 (arg1, arg2, arg3, arg4, sch_errno);
-  
-  sch_errno = s48_enter_integer(the_errno);
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 6,
-                             sch_errno,
+			     s48_enter_fixnum(the_errno),
 			     s48_enter_string(strerror(the_errno)),
 			     arg1, arg2, arg3, arg4);
-  S48_GC_UNPROTECT();
 }
 
 void
 s48_raise_os_error_5(int the_errno, s48_value arg1, s48_value arg2, 
 		     s48_value arg3, s48_value arg4, s48_value arg5) {
-
-  s48_value sch_errno = S48_UNSPECIFIC;
-  
-  S48_DECLARE_GC_PROTECT(6);
-  S48_GC_PROTECT_6 (arg1, arg2, arg3, arg4, arg5, sch_errno);
-  
-  sch_errno = s48_enter_integer(the_errno);
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 7,
-                             sch_errno,
+			     s48_enter_fixnum(the_errno),
 			     s48_enter_string(strerror(the_errno)),
 			     arg1, arg2, arg3, arg4, arg5);
-  S48_GC_UNPROTECT();
 }
 
 void
 s48_raise_os_error_6(int the_errno, s48_value arg1, s48_value arg2, 
 		     s48_value arg3, s48_value arg4, s48_value arg5,
 		     s48_value arg6) {
-
-  s48_value sch_errno = S48_UNSPECIFIC;
-  
-  S48_DECLARE_GC_PROTECT(7);
-  S48_GC_PROTECT_7 (arg1, arg2, arg3, arg4, arg5, arg6, sch_errno);
-  
-  sch_errno = s48_enter_integer(the_errno);
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 8,
-                             sch_errno,
+			     s48_enter_fixnum(the_errno),
 			     s48_enter_string(strerror(the_errno)),
 			     arg1, arg2, arg3, arg4, arg5, arg6);
-  S48_GC_UNPROTECT();
 }
 
 void
 s48_raise_os_error_7(int the_errno, s48_value arg1, s48_value arg2, 
 		     s48_value arg3, s48_value arg4, s48_value arg5,
 		     s48_value arg6, s48_value arg7) {
-
-  s48_value sch_errno = S48_UNSPECIFIC;
-  
-  S48_DECLARE_GC_PROTECT(8);
-  S48_GC_PROTECT_8 (arg1, arg2, arg3, arg4, arg5, arg6, arg7, sch_errno);
-  
-  sch_errno = s48_enter_integer(the_errno);
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 9,
-                             sch_errno,
+			     s48_enter_fixnum(the_errno),
 			     s48_enter_string(strerror(the_errno)),
 			     arg1, arg2, arg3, arg4, arg5, arg6, arg7);
-  S48_GC_UNPROTECT();
 }
 
 void
@@ -1173,7 +1114,7 @@ s48_value
 s48_enter_substring(char *str, int length)
 {
   s48_value obj = s48_allocate_stob(S48_STOBTYPE_STRING, length + 1);
-  memcpy(S48_UNSAFE_EXTRACT_STRING(obj), str, length);
+  strncpy(S48_UNSAFE_EXTRACT_STRING(obj), str, length);
   *(S48_UNSAFE_EXTRACT_STRING(obj) + length) = '\0';
   return obj;
 }
