@@ -9,8 +9,8 @@
 ** the top. -Olin
 */
 #if !defined(DEFAULT_HEAP_SIZE)
-/* 5 megacell = 20 megabytes (10 meg per semispace) */
-#define DEFAULT_HEAP_SIZE 5000000L
+/* 4 megacell = 16 megabytes (8 meg per semispace) */
+#define DEFAULT_HEAP_SIZE 4500000L
 #endif
 
 #if !defined(DEFAULT_STACK_SIZE)
@@ -24,7 +24,7 @@
 
 /* DEFAULT_IMAGE_NAME should be defined using the -D switch to cc. */
 #if !defined(DEFAULT_IMAGE_NAME)
-#define DEFAULT_IMAGE_NAME "scsh.image"
+#define DEFAULT_IMAGE_NAME "scheme48.image"
 #endif
 
 #endif /* STATIC_AREAS */
@@ -39,7 +39,7 @@ char ** process_args(char **argv,
 
 extern int 
 internal_s48_main(long heap_size, long stack_size, 
-		  char* object_file, char* image_name, 
+		  char* prog_name, char* object_file, char* image_name, 
 		  int argc, char** argv);
 
 int
@@ -62,13 +62,12 @@ main(argc, argv)
 #endif
 
   long vm_argc = 0;
-  prog_name = *argv++;/* Save program name. */
+  char *me = *argv;		/* Save program name. */
+  prog_name = *argv++;
 
-  argv=process_args(argv,
+  argv=process_args(argv, 		
 		    &heap_size, &stack_size,
 		    &object_file, &image_name);
-  argv--;
-  argv[0] = prog_name;
   for(argc=0, argp=argv; *argp; argc++, argp++); /* Recompute argc. */
-  return internal_s48_main(heap_size, stack_size, object_file, image_name, argc, argv);
+  return internal_s48_main(heap_size, stack_size, prog_name, object_file, image_name, argc, argv);
 }
