@@ -1,7 +1,6 @@
 /* Copyright (c) 1993-1999 by Richard Kelsey and Jonathan Rees.
    See file COPYING. */
 
-#include "sysdep.h"
 #include <signal.h>		/* for sigaction() (POSIX.1) */
 #include <stdlib.h>
 #include <unistd.h>
@@ -10,6 +9,7 @@
 #include <sys/time.h>
 #include <sys/times.h>
 #include <errno.h>              /* for errno, (POSIX?/ANSI) */
+#include "sysdep.h"
 #include "c-mods.h"
 #include "scheme48vm.h"
 #include "event.h"
@@ -30,10 +30,7 @@ static void     when_sigpipe_interrupt();
 /* JMG:*/
 static void     when_scsh_interrupt();
 /* JMG: for scsh */
-#define INTERRUPT_QUEUE_LENGTH 32
-
-static int      interrupt_queue [INTERRUPT_QUEUE_LENGTH];
-static int	next_interrupt = 0;
+static long interrupt_count[32];
 static int s48_os_signal_pending(void);
 static bool s48_os_signal_happend(void);
 
@@ -45,6 +42,7 @@ void		s48_start_alarm_interrupts(void);
 void
 s48_sysdep_init(void)
 {
+  int i;
   if (!s48_setcatcher(SIGINT, when_keyboard_interrupt)
       || !s48_setcatcher(SIGALRM, when_alarm_interrupt)
       || !s48_setcatcher(SIGPIPE, when_sigpipe_interrupt)) {
@@ -53,6 +51,8 @@ s48_sysdep_init(void)
 	    errno);
     exit(1);
   }
+  for (i = 0; i < max_sig; i++)
+    interrupt_count[i] = 0;
 
   sigfillset (&full_sigset);
       
@@ -286,7 +286,7 @@ s48_stop_alarm_interrupts(void)
  *                  (queue-ready-ports)
  *                  (set! *poll-time* (+ *time* *poll-interval*))))
  *           (cond ((not (queue-empty? ready-ports))
- *                  (values (enum event-type i/o-{read/write}-completion)
+ *                  (values (enum event-type i/o-completion)
  *                          (dequeue! ready-ports)))
  *                 ((>= *current_time* *alarm-time*)
  *                  (set! *alarm-time* max-integer)
@@ -301,20 +301,9 @@ s48_stop_alarm_interrupts(void)
  *                  (values (enum event-type no-event) #f))))))
  */
 
-#define FD_QUIESCENT 0			/* idle */
-#define FD_READY     1			/* I/O ready to be performed */
-#define FD_PENDING   2			/* waiting */
-
-typedef struct fd_struct {
- int	fd,			/* file descriptor */
-	status;			/* one of the FD_* constants */
- bool	is_input;		/* iff input */
- struct fd_struct	*next;	/* next on same queue */
-} fd_struct;
-
-static bool	 there_are_ready_ports(void);
-static fd_struct *next_ready_fd_struct(void);
-static int	 queue_ready_ports(bool wait, long seconds, long ticks);
+static bool	there_are_ready_ports(void);
+static int	next_ready_port(void);
+static int	queue_ready_ports(bool wait, long seconds, long ticks);
 
 int
 s48_get_next_event(long *ready_fd, long *status)
@@ -324,8 +313,6 @@ s48_get_next_event(long *ready_fd, long *status)
   */
 
   int io_poll_status;
-  fd_struct *f;
-
   /*
     fprintf(stderr, "[poll at %d (waiting for %d)]\n", s48_current_time, alarm_time);
     */
@@ -346,14 +333,10 @@ s48_get_next_event(long *ready_fd, long *status)
     }
   }
   if (there_are_ready_ports()) {
-    f = next_ready_fd_struct();
-    *ready_fd = f->fd;
+    *ready_fd = next_ready_port();
     *status = 0;   /* chars read or written */
     /* fprintf(stderr, "[i/o completion]\n"); */
-    if (f->is_input)
-      return (IO_READ_COMPLETION_EVENT);
-    else
-      return (IO_WRITE_COMPLETION_EVENT);
+    return (IO_COMPLETION_EVENT);
   }
   if (alarm_time != -1 && s48_current_time >= alarm_time) {
     alarm_time = -1;
@@ -380,6 +363,17 @@ s48_get_next_event(long *ready_fd, long *status)
  * the pending ports and move any that are ready onto the other queue and
  * signal an event.
  */
+#define FD_QUIESCENT 0			/* idle */
+#define FD_READY     1			/* I/O ready to be performed */
+#define FD_PENDING   2			/* waiting */
+
+typedef struct fd_struct {
+ int	fd,			/* file descriptor */
+	status;			/* one of the FD_* constants */
+ bool	is_input;		/* iff input */
+ struct fd_struct	*next;	/* next on same queue */
+} fd_struct;
+
 
 /*
  * A queue of fd_structs is empty iff the first field is NULL.  In
@@ -464,14 +458,14 @@ there_are_ready_ports(void)
 }
 
 
-static fd_struct *
-next_ready_fd_struct(void)
+static int
+next_ready_port(void)
 {
   fd_struct	*p;
 
   p = rmque(&ready.first, &ready);
   p->status = FD_QUIESCENT;
-  return (p);
+  return (p->fd);
 }
 
 
@@ -625,10 +619,11 @@ queue_ready_ports(bool wait, long seconds, long ticks)
   tvp = &tv;
   if (wait)
     if (seconds == -1){
-      tvp = NULL;
+      tv.tv_sec = 1;
+      tv.tv_usec = 0;
     }
     else {
-      tv.tv_sec = seconds;
+      tv.tv_sec = (seconds > 0) ? 1 : 0;
       tv.tv_usec = ticks * (1000000 / TICKS_PER_SECOND);
     }
   else
@@ -653,6 +648,18 @@ queue_ready_ports(bool wait, long seconds, long ticks)
 	poll_time = -1;
       return NO_ERRORS;
     }
+    else if (wait && (left == 0) && (limfd == 0)){
+      if (seconds > 1){
+	seconds--;
+	tv.tv_sec = 1; /* select maybe destroyed tv */
+	tv.tv_usec = 0; /* we already waited the usecs */
+      }
+      else if (seconds > -1) return NO_ERRORS;
+      else { /* loop if seconds == -1 */
+	tv.tv_sec = 1;
+	tv.tv_usec = 0;
+      }
+    }
     else if (left == 0)
       return NO_ERRORS;
     else if (errno == EINTR) {
@@ -664,25 +671,11 @@ queue_ready_ports(bool wait, long seconds, long ticks)
   }
 }
 
-/*
- * Adds `signum' to the queue of received signals.
- */
-
-static void
-queue_interrupt(int signum)
-{
-  if (next_interrupt == INTERRUPT_QUEUE_LENGTH){
-    perror("Interrupt queue overflow -- report to Scheme 48 maintainers.");
-    exit(-1); 
-  }
-  interrupt_queue[next_interrupt] = signum;
-  next_interrupt++;
-}
 
 /* JMG: for scsh */
 static void when_scsh_interrupt(int signo)
 {
-  queue_interrupt(sig2int[signo]);
+  interrupt_count[sig2int[signo]] +=1;
   NOTE_EVENT;
   return;
 }
@@ -710,34 +703,31 @@ static void when_scsh_interrupt(int signo)
  * reenabled when the handler returns (or if done by hand).
  */
 
-/*
- * Returns TRUE if there is a signal to be delivered up to Scheme.
- * Needs no be called with interrupts blocked.
- */
-
+/* needs no be called with interrupts blocked */
 int
 s48_os_signal_pending(void) {
   int i;
-  s48_value interrupt_list = S48_NULL;
-  block_interrupts();
 
-  if (next_interrupt == 0) {
-    allow_interrupts();
-    return FALSE; }
-  else {
-    /* turn the queue into a scheme list and preserve the order */
-    for (i = next_interrupt; i > 0 ; i--)
-      interrupt_list = s48_cons (s48_enter_fixnum (interrupt_queue  [i - 1]),
-				 interrupt_list);
-    s48_set_os_signals(interrupt_list);
-		      
-    next_interrupt = 0;
-    allow_interrupts();
-    return TRUE; }
+  for (i = 0; i < max_sig; i++){
+    if (interrupt_count[i] > 0){
+      --interrupt_count[i];
+      allow_interrupts();
+      s48_set_os_signal(S48_UNSAFE_ENTER_FIXNUM(i),
+			S48_UNSAFE_ENTER_FIXNUM(0));
+      return TRUE;
+    }
+  }
+  return FALSE;
 }
 
 bool 
 s48_os_signal_happend(void) {
-  return (next_interrupt != 0);
+  int i;
+  for (i = 0; i < max_sig; i++){
+    if (interrupt_count[i] > 0){
+      return TRUE;
+    }
+  }
+  return FALSE;
 }
 
