@@ -37,30 +37,27 @@
 ;;; Abstract out common code for create-{directory,fifo,hard-link,symlink}:
 
 (define (create-file-thing fname makeit override? op-name syscall)
-    (let ((query (lambda ()
-		   (y-or-n? (string-append op-name ": " fname
-					   " already exists. Delete")))))
-      (let ((result
-	     (let loop ((override? override?))
-	       (with-errno-handler
-		((err data)
-		 ((errno/exist)
-		  (cond ((if (eq? override? 'query)
-			     (query)
-			     override?)
-			 (delete-filesys-object fname)
-			 (loop #t))
+  (let ((query (lambda ()
+		 (y-or-n? (string-append op-name ": " fname
+					 " already exists. Delete")))))
+    (let ((result
+	   (let loop ((override? override?))
+	     (with-errno-handler
+	      ((err data)
+	       ((errno/exist)
+		(cond ((if (eq? override? 'query)
+			   (query)
+		       override?)
+		       (delete-filesys-object fname)
+		       (loop #t))
 		      ;;; raising an error here won't work due to S48's
 		      ;;; broken exception system
-			(else (list err syscall fname)))))
-		(with-resources-aligned 
-		 (list cwd-resource umask-resource euid-resource egid-resource)
-		 (lambda ()
-		   (makeit fname)))
-		#f))))
-	(if (list? result)
-	    (apply errno-error result)
-	    (if #f #f)))))
+		      (else (list err syscall fname)))))
+	      (makeit fname)
+	      #f))))
+      (if (list? result)
+	  (apply errno-error result)
+	  (if #f #f)))))
 
 ;;;;;;;
 
@@ -115,20 +112,4 @@
 	    (and override?
 		 (y-or-n? (string-append "rename-file:" new-fname
 					 " already exists. Delete"))))
-	(with-resources-aligned 
-	 (list cwd-resource euid-resource egid-resource)
-	 (lambda ()
-	   (%rename-file old-fname new-fname))))))
-
-(define (read-symlink path)
-  (with-resources-aligned 
-	 (list cwd-resource euid-resource egid-resource)
-	 (lambda ()
-	   (%read-symlink path))))
-
-
-(define (delete-directory path)
-  (with-resources-aligned 
-	 (list cwd-resource euid-resource egid-resource)
-	 (lambda ()
-	   (%delete-directory path))))
\ No newline at end of file
+	(%rename-file old-fname new-fname))))
