@@ -204,7 +204,7 @@
 		   "bind-socket: trying to bind incompatible address to socket ~s"
 		   name))
 		 ((and (= family address-family/unix)
-		       (> (string-length name) 107))
+		       (> (string-length (socket-address->unix-address name)) 107))
 		  (error "bind-socket: path too long" name))
 		 (else
 		  (%bind (socket->fdes sock)
@@ -239,17 +239,19 @@
 				   (fdport-data:channel 
 				    (fdport-data (socket:inport sock))))
 				  (if (eq? res 0)
-				      (handle-EINPROGRESS sock)
+				      ;; If connect returned EINPROGRESS, we can check 
+				      ;; it's success after  the next success with getsockopt
+				      (let ((val (socket-option sock 
+								level/socket 
+								socket/error)))
+					(if (not (zero? val))
+					    (errno-error val 
+							 (errno-msg val) 
+							 %connect 
+							 sock 
+							 name)))
 				      (loop))))))))))))
 
-;;; If connect returned EINPROGRESS, we can check it's success after 
-;;; the next success with getsockopt
-
-(define (handle-EINPROGRESS sock)
-  (let ((val (socket-option sock level/socket socket/error)))
-    (if (not (zero? val))
-	(errno-error val "scheme_connect"))))
-
 (import-os-error-syscall %connect (sockfd family name) "scheme_connect")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
