@@ -83,25 +83,33 @@ s48_value scheme_connect(s48_value sock, s48_value family, s48_value scheme_name
     case AF_UNIX: 
       {
 	struct sockaddr_un name;
+	int ret;
 	int scheme_length=S48_STRING_LENGTH(scheme_name);
 	
 	memset(&name, 0, sizeof(name));
 	
 	name.sun_family=AF_UNIX;	
-
+	if (scheme_length>=(108-1)) /* save space for \0 */
+	  return(-1);
 	strncpy(name.sun_path,
 		s48_extract_string (scheme_name),
 		scheme_length);	/* copy to c string */
 	name.sun_path[scheme_length]='\0'; /* add null */
 	
 	if (connect(sockfd,(struct sockaddr *)&name,sizeof(name)) == 0)
-	  return s48_cons (S48_FALSE, S48_UNSPECIFIC);
+	  return S48_TRUE;
 	
 	if (errno != EWOULDBLOCK && errno != EINTR && errno != EALREADY
 	    && errno != EINPROGRESS && errno != EAGAIN)
 	  s48_raise_os_error_3(errno, sock, family, scheme_name);
 	 
-	return s48_cons (S48_TRUE, (errno == EINPROGRESS) ? S48_TRUE : S48_FALSE);	
+	if (! (s48_add_pending_fd(sockfd, 0)))
+	  s48_raise_out_of_memory_error();
+	
+	if (errno == EINPROGRESS) 
+	  return s48_enter_fixnum (0); 
+	else return S48_FALSE;
+	
       }
     case AF_INET: 
       {
@@ -117,13 +125,19 @@ s48_value scheme_connect(s48_value sock, s48_value family, s48_value scheme_name
 	name.sin_port=port;
 	
 	if (connect(sockfd,(struct sockaddr *)&name,sizeof(name)) == 0)
-	  return s48_cons (S48_FALSE, S48_UNSPECIFIC);
+	  return S48_TRUE;
 	
 	if (errno != EWOULDBLOCK && errno != EINTR && errno != EALREADY
 	    && errno != EINPROGRESS && errno != EAGAIN)
 	  s48_raise_os_error_3(errno, sock, family, scheme_name);
+
+	if (! (s48_add_pending_fd(sockfd, 0)))
+	  s48_raise_out_of_memory_error();
+	
+	if (errno == EINPROGRESS) 
+	  return s48_enter_fixnum (0); 
+	else return S48_FALSE;
 	
-	return s48_cons (S48_TRUE, (errno == EINPROGRESS) ? S48_TRUE : S48_FALSE);
       }
     default: 
       s48_raise_argument_type_error (family);	/* error unknown address family */ 
@@ -159,9 +173,8 @@ s48_value scheme_accept(s48_value sockfd_tagged, s48_value family)
 	      s48_raise_out_of_memory_error();
 	    return S48_FALSE;
 	  }
-	fcntl(newsockfd, F_SETFL, O_NONBLOCK);
-	return(s48_cons (s48_enter_fixnum (newsockfd),
-			 s48_enter_string (name.sun_path)));
+	
+	return(s48_enter_fixnum (newsockfd));
 	break;
       }
     case AF_INET: 
@@ -292,6 +305,7 @@ recv_substring(s48_value scm_sockfd, s48_value flags, s48_value buf,
 		  buf_part, end-start, 
 		  s48_extract_fixnum (flags),
 		  (struct sockaddr *)&name, &namelen);
+  s48_value result;
   
   if (cc >= 0)
     return (s48_cons (s48_enter_fixnum (cc), 
@@ -321,12 +335,12 @@ s48_value send_substring(s48_value scm_sockfd,
   int flags = s48_extract_fixnum (scm_flags);
   int start = s48_extract_fixnum (scm_start);
   int end = s48_extract_fixnum (scm_end);
+  char* buf_part = s48_extract_string (buf) + start;
   
   switch(s48_extract_fixnum (scm_family))
     {
     case 0:			/* only with connected sockets */
       {
-        char* buf_part = s48_extract_string (buf) + start;
 	n = send(s, buf_part, end-start, flags);
 	break;
       }
@@ -334,7 +348,6 @@ s48_value send_substring(s48_value scm_sockfd,
       {
 	struct sockaddr_un name;
 	int scheme_length=S48_STRING_LENGTH(scheme_name);
-        char* buf_part = s48_extract_string (buf) + start;
 
 	memset(&name, 0, sizeof(name));
 
@@ -354,16 +367,8 @@ s48_value send_substring(s48_value scm_sockfd,
     case AF_INET:
       {
 	struct sockaddr_in name;
-	u_long addr;
-	u_short port;
-        char* buf_part;
-        S48_DECLARE_GC_PROTECT(2);
-
-        S48_GC_PROTECT_2 (scheme_name,buf);
-          
-        addr = htonl (s48_extract_unsigned_integer (S48_CAR (scheme_name)));
-        port = htons (s48_extract_fixnum (S48_CDR (scheme_name)));
-        buf_part = s48_extract_string (buf) + start;
+	u_long  addr = htonl (s48_extract_unsigned_integer (S48_CAR (scheme_name)));
+	u_short port = htons(s48_extract_fixnum (S48_CDR (scheme_name)));
 
 	memset(&name, 0, sizeof(name));
 
@@ -381,7 +386,7 @@ s48_value send_substring(s48_value scm_sockfd,
       s48_raise_argument_type_error (s48_extract_fixnum (scm_family));	
       /* error unknown address family */
     }
-  S48_GC_UNPROTECT();
+  
   if (n >= 0)
     return s48_enter_fixnum (n);
 
@@ -409,7 +414,7 @@ s48_value scheme_getsockopt (s48_value s,
 			     s48_value optname)
 {
   int optval;
-  size_t optlen=sizeof(optval);
+  int optlen=sizeof(optval);
   
   if (getsockopt(s48_extract_fixnum (s),
 		 s48_extract_fixnum (level),
@@ -426,7 +431,7 @@ s48_value scheme_getsockopt_linger (s48_value s,
 				    s48_value optname)
 {
   struct linger optval;
-  size_t optlen=sizeof(optval);
+  int optlen=sizeof(optval);
   
   if (getsockopt(s48_extract_fixnum (s),
 		 s48_extract_fixnum (level),
