@@ -1,6 +1,7 @@
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+
+extern char *prog_name;
 
 #define streq(a,b) (strcmp((a),(b))==0)
 
@@ -15,13 +16,12 @@ static void usage(void) {
             "end-option: -i <image file name>\n"
             "            -- (Terminates vm args.)\n"
             "            -a (Terminates vm args. Obsolete.)\n",
-            "scshvm");}
+            prog_name);}
 
-static void bad_args(int reason) {
-  /* fprintf(stderr, "reason : %s\n", reason); */
-  usage();
-  exit(1); 
-}
+static void bad_args(int nr) {
+  fprintf(stderr, "reason : %d\n", nr);
+    usage();
+    exit(1); }
 
 char ** process_args(char **argv,
                      long *pheap_size,
@@ -37,18 +37,14 @@ char ** process_args(char **argv,
         argv = process_meta_arg(argv);
         if( !argv ) {
             fprintf(stderr, "%s: \\ <fname> expansion failed.\n",
-                    "scshvm");
+                    prog_name);
             exit(1);}}
 
     for (; *argv; argv++)
-         if (streq (argv[0],""))
-           /* process_meta_arg inserts an empty argument for every space. 
-              Skip it. */
-           continue;
-         else if( argv[0][0] != '-' )
-           bad_args(1); /* die */
-         else
-           switch (argv[0][1]) {
+        if( argv[0][0] != '-' )
+            bad_args(1); /* die */
+        else
+            switch (argv[0][1]) {
             default:
                 bad_args(2); /* die */
                 break;
@@ -56,7 +52,6 @@ char ** process_args(char **argv,
             case 'h': /* heapsize */
                 argv++;
                 if( !*argv ) bad_args(3); /* die */
-                while (streq (argv[0],"")) argv++; /* Skip empty arguments */
                 *pheap_size = atoi(*argv);
                 if( *pheap_size <= 0 ) bad_args(4);
                 break;
@@ -64,7 +59,6 @@ char ** process_args(char **argv,
             case 's':
                 argv++;
                 if( !*argv ) bad_args(5); /* die */
-                while (streq (argv[0],"")) argv++; /* Skip empty arguments */
                 *pstack_size = atoi(*argv);
                 if (*pstack_size <= 0) bad_args(6);
                 break;
@@ -72,7 +66,6 @@ char ** process_args(char **argv,
             case 'o': /* object file */
                 argv++;
                 if( !*argv ) bad_args(7); /* die */
-                while (streq (argv[0],"")) argv++; /* Skip empty arguments */
                 *pobject_file = *argv;
                 break;
 
@@ -80,7 +73,6 @@ char ** process_args(char **argv,
             case 'i':
                 argv++;
                 if( !*argv ) bad_args(8); /* die */
-                while (streq (argv[0],"")) argv++; /* Skip empty arguments */
                 *pimage_name = *argv++;
                 return argv;
 
