@@ -6,94 +6,79 @@
 ;;; A clever definition in a clever implementation allows the caller's stack
 ;;; and dynamic env to be gc'd away, since this procedure never returns.
 
-(define (call-terminally thunk)
-  (with-continuation
-   null-continuation
-   (lambda ()
-     (with-handler 
-      (lambda (c more)
-        (display-condition c (current-error-port))
-        (exit 1))
-     (lambda ()
-       (dynamic-wind
-        (lambda () (values))
-        thunk
-        (lambda () (exit 0))))))))
+;;;(define (call-terminally thunk)
+;;;  (with-continuation (lambda () #f) (lambda () (thunk) (exit 0))))
+;;;  ;; Alternatively: (with-continuation #f thunk)
 
-;; from shift-reset.scm:
-(define null-continuation #f)
+;;; More portably, but less usefully:
+;;; New version of s48 requires with-continuation to take a continuation
+(define (call-terminally thunk)
+  (thunk)
+  (exit 0))
 
 ;;; Like FORK, but the parent and child communicate via a pipe connecting
 ;;; the parent's stdin to the child's stdout. This function side-effects
 ;;; the parent by changing his stdin.
 
-(define (fork/pipe . stuff)
-  (really-fork/pipe fork stuff))
+(define (fork/pipe . maybe-thunk)
+  (really-fork/pipe fork maybe-thunk))
 
-(define (%fork/pipe . stuff)
-  (really-fork/pipe %fork stuff))
+(define (%fork/pipe . maybe-thunk)
+  (really-fork/pipe %fork maybe-thunk))
   
 ;;; Common code for FORK/PIPE and %FORK/PIPE.
-(define (really-fork/pipe forker rest)
-  (let-optionals rest ((maybe-thunk #f)
-		       (no-new-command-level? #f))
-    (receive (r w) (pipe)
-      (let ((proc (forker #f no-new-command-level?)))
-	(cond (proc		; Parent
-	       (close w)
-	       (move->fdes r 0)
-               (set-current-input-port! r))
-	      (else		; Child
-	       (close r)
-               (move->fdes w 1)
-               (if maybe-thunk
-                   (with-current-output-port
-                    w
-                    (call-terminally maybe-thunk))
-                   (set-current-output-port! w))))
-	proc))))
+(define (really-fork/pipe forker maybe-thunk)
+  (receive (r w) (pipe)
+    (let ((proc (forker)))
+      (cond (proc		; Parent
+	     (close w)
+	     (move->fdes r 0))
+	    (else		; Child
+	     (close r)
+	     (move->fdes w 1)
+	     (if (pair? maybe-thunk)
+		 (call-terminally (car maybe-thunk)))))
+      proc)))
 
 
 ;;; FORK/PIPE with a connection list.
 ;;; (FORK/PIPE . m-t) = (apply fork/pipe+ '((1 0)) m-t)
 
-(define (%fork/pipe+ conns . stuff)
-  (really-fork/pipe+ %fork conns stuff))
+(define (%fork/pipe+ conns . maybe-thunk)
+  (really-fork/pipe+ %fork conns maybe-thunk))
 
-(define (fork/pipe+ conns . stuff)
-  (really-fork/pipe+ fork conns stuff))
+(define (fork/pipe+ conns . maybe-thunk)
+  (really-fork/pipe+ fork conns maybe-thunk))
 
 ;;; Common code.
-(define (really-fork/pipe+ forker conns rest)
-  (let-optionals rest ((maybe-thunk #f)
-		       (no-new-command-level? #f))
-    (let* ((pipes (map (lambda (conn) (call-with-values pipe cons))
-		       conns))
-	   (rev-conns (map reverse conns))
-	   (froms (map (lambda (conn) (reverse (cdr conn)))
-		       rev-conns))
-	   (tos (map car rev-conns)))
+(define (really-fork/pipe+ forker conns maybe-thunk)
+  (let* ((pipes (map (lambda (conn) (call-with-values pipe cons))
+		     conns))
+	 (rev-conns (map reverse conns))
+	 (froms (map (lambda (conn) (reverse (cdr conn)))
+		     rev-conns))
+	 (tos (map car rev-conns)))
 
-      (let ((proc (forker #f no-new-command-level?)))
-	(cond (proc			; Parent
-	       (for-each (lambda (to r/w)
-			   (let ((w (cdr r/w))
-				 (r (car r/w)))
-			     (close w)
-			     (move->fdes r to)))
-			 tos pipes))
+    (let ((proc (forker)))
+      (cond (proc			; Parent
+	     (for-each (lambda (to r/w)
+			 (let ((w (cdr r/w))
+			       (r (car r/w)))
+			   (close w)
+			   (move->fdes r to)))
+		       tos pipes))
 
-	      (else			; Child
-	       (for-each (lambda (from r/w)
-			   (let ((r (car r/w))
-				 (w (cdr r/w)))
-			     (close r)
-			     (for-each (lambda (fd) (dup w fd)) from)
-			     (close w))) ; Unrevealed ports win.
-			 froms pipes)
-	       (if maybe-thunk
-		   (call-terminally maybe-thunk))))
-	proc))))
+	    (else		; Child
+	     (for-each (lambda (from r/w)
+			 (let ((r (car r/w))
+			       (w (cdr r/w)))
+			   (close r)
+			   (for-each (lambda (fd) (dup w fd)) from)
+			   (close w))) ; Unrevealed ports win.
+		       froms pipes)
+	     (if (pair? maybe-thunk)
+		 (call-terminally (car maybe-thunk)))))
+      proc)))
 
 (define (tail-pipe a b)
   (fork/pipe a)
@@ -132,302 +117,146 @@
 (define (with-lock lock thunk)
   (dynamic-wind 
    (lambda ()
-     (obtain-lock lock))
+     (release-lock lock))
    thunk
    (lambda ()
      (release-lock lock))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+
+;;; A resource is a part of the process state for which every thread
+;;; has its own value
+;;; uses the procedures:
+;;; (process-read-resource (-> 'X))
+;;; (process-set-resource ('X -> unspec))
+;;; (resource-eq? ('X 'X -> bool))
+
+;;; defines the procedures:
+;;; (initialize-resource  (-> unspec)) ; call on startup
+;;; (with-resource* ((-> 'X) -> 'X))
+;;; (with-resource-aligned* ((-> 'X) -> 'X))
+;;; (thread-read-resource (-> 'X))
+;;; (thread-set-resource ('X -> unspec))
+
+(define-syntax  make-process-resource
+  (syntax-rules 
+   ()
+   ((make-process-resource 
+     initialize-resource 
+     thread-read-resource thread-set-resource! thread-change-resource 
+     with-resource* with-resource-aligned* 
+     process-read-resource process-set-resource resource-eq?)
+    (begin 
+      (define *resource-cache* 'uninitialized)
+      (define resource-lock 'uninitialized)
+       
+      (define (initialize-resource)
+	(set! *resource-cache* (process-read-resource))
+	(set! $resource ;;; TODO The old thread-fluid will remain
+	      (make-preserved-thread-fluid
+	       (process-read-resource)))
+	(set! resource-lock (make-lock)))
+       
+      (define (cache-value)
+	*resource-cache*)
+       
+      ;; Actually do the syscall and update the cache
+      ;; assumes the resource lock obtained
+      (define (change-and-cache dir)
+	(process-set-resource dir)
+	(set! *resource-cache* (process-read-resource)))
+      
+      ;; The thread-specific resource: A thread fluid
+      
+      (define $resource 'empty-resource-value)
+       
+      (define (thread-read-resource) (thread-fluid $resource))
+      (define (thread-set-resource! dir) (set-thread-fluid! $resource dir))
+      (define (let-resource dir thunk)
+	(let-thread-fluid $resource dir thunk))
+       
+      (define (with-resource* dir thunk)
+	(let ((changed-dir #f))		
+	  (with-lock resource-lock
+		     (lambda ()
+		       (align-resource!)
+		       (change-and-cache dir)
+		       (set! changed-dir (cache-value))))
+	  (let-resource changed-dir thunk)))
+       
+      ;; Align the value of the Unix resource with scsh's value.
+      ;; Since another thread could disalign, this call and
+      ;; any ensuring syscall that relies upon it should
+      ;; be "glued together" with the resource lock.
+      
+      (define (align-resource!)
+	(let ((dir (thread-read-resource)))
+	  (if (not (resource-eq? dir (cache-value)))
+	      (change-and-cache dir))))
+
+      (define (thread-change-resource dir)
+	(with-lock resource-lock
+		   (lambda ()
+		     (align-resource!)
+		     (change-and-cache dir)
+		     (thread-set-resource! (cache-value)))))
+
+      ;; For thunks that don't raise exceptions or throw to continuations,
+      ;; this is overkill & probably a little heavyweight for frequent use.
+      ;; But it is general.
+      ;;
+      ;; A less-general, more lightweight hack could be done just for
+      ;; syscalls.  We could probably dump the DYNAMIC-WINDs and build the
+      ;; rest of the pattern into one of the syscall-defining macros, or
+      ;; something.  
+      ;; Olin adds the following: the efficient way to do things is not
+      ;; with a dynamic wind or a lock. Just turn off interrupts, sync the
+      ;; resource, do the syscall, turn them back on.
+
+      (define (with-resource-aligned* thunk)
+	(dynamic-wind (lambda ()
+			(with-lock resource-lock
+				   align-resource!)) 
+		      thunk 
+		      values))
+
+      ;; example syscall
+      ;; (define (exported-delete-file fname)
+      ;;  (with-cwd-aligned (really-delete-file fname)))
+
+
+      (define resource-reinitializer
+	(make-reinitializer (lambda () (warn "calling resumer") (initialize-resource))))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; working directory per thread
-
-(define *cwd-cache* 'uninitialized)
-(define cwd-lock (make-lock))
-
-(define (initialize-cwd)
-  (set! *cwd-cache* (process-cwd))
-  (set! $cwd ;;; TODO The old thread-fluid will remain
-	(make-preserved-thread-fluid
-	 (cwd-cache))))
-;  (set! cwd-lock (make-lock)))
-
-(define (cwd-cache)
-  *cwd-cache*)
-
-;; Actually do the syscall and update the cache
-;; assumes the cwd lock obtained
-(define (change-and-cache-cwd new-cwd)
-  (if (not (file-name-absolute? new-cwd))
-      (process-chdir (string-append (cwd) "/" new-cwd))
-      (process-chdir new-cwd))
-  (set! *cwd-cache* (process-cwd)))
-
-;; The thread-specific cwd: A thread fluid
-
-(define $cwd 'empty-cwd-value)
-
-(define (cwd) (thread-fluid $cwd))
-(define (thread-set-cwd! cwd) (set-thread-fluid! $cwd cwd))
-(define (let-cwd cwd thunk)
-  (let-thread-fluid $cwd cwd thunk))
-
-(define (with-cwd* new-cwd thunk)
-  (let ((changed-cwd
-	 (with-lock cwd-lock
-	   (lambda ()
-	     (change-and-cache-cwd new-cwd)
-	     (cwd-cache)))))
-    (let-cwd changed-cwd thunk)))
-
-;; Align the value of the Unix cwd with scsh's value.
-;; Since another thread could disalign, this call and
-;; any ensuring syscall that relies upon it should
-;; be "glued together" with the cwd lock.
-
-(define (align-cwd!)
-  (let ((thread-cwd (cwd)))
-    (if (not (string=? thread-cwd (cwd-cache)))
-	(change-and-cache-cwd thread-cwd))))
-
-(define (chdir . maybe-dir)
-  (let ((dir (:optional maybe-dir (home-dir))))
-    (with-lock cwd-lock
-      (lambda ()
-	(change-and-cache-cwd dir)
-	(thread-set-cwd! (cwd-cache))))))
-
-(define-record-type resource :resource
-  (make-resource align! lock)
-  resource?
-  (align! resource-align!)
-  (lock resource-lock))
-
-(define (with-resources-aligned resources thunk)
-   (let ((locks (map resource-lock resources)))
-     (apply obtain-all-or-none locks)
-     (for-each
-      (lambda (align!) (align!))
-      (map resource-align! resources))
-     (let ((val (with-handler
-		 (lambda (cond more)
-		   (for-each release-lock locks)
-		   (more))
-		 thunk)))
-       (for-each release-lock locks)
-       val)))
-
-(define cwd-resource (make-resource align-cwd! cwd-lock))
-
-;; example syscall
-;; (define (exported-delete-file fname)
-;;  (with-cwd-aligned (really-delete-file fname)))
-
-(define cwd-reinitializer
-  (make-reinitializer (lambda () (initialize-cwd))))
-
+(make-process-resource 
+ initialize-cwd cwd thread-set-cwd! chdir with-cwd* with-cwd-aligned*
+ process-cwd process-chdir string=?)
 
 (initialize-cwd)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; umask per thread
-(define *umask-cache* 'uninitialized)
-(define umask-lock (make-lock))
-
-(define (initialize-umask)
-  (set! *umask-cache* (process-umask))
-  (set! $umask ;;; TODO The old thread-fluid will remain
-	(make-preserved-thread-fluid
-	 (umask-cache))))
-;  (set! umask-lock (make-lock)))
-
-(define (umask-cache)
-  *umask-cache*)
-
-;; Actually do the syscall and update the cache
-;; assumes the resource lock obtained
-(define (change-and-cache-umask new-umask)
-  (set-process-umask new-umask)
-  (set! *umask-cache* (process-umask)))
-
-;; The thread-specific umask: A thread fluid
-
-(define $umask 'empty-umask-value)
-
-(define (umask) (thread-fluid $umask))
-(define (thread-set-umask! new-umask) (set-thread-fluid! $umask new-umask))
-(define (let-umask new-umask thunk)
-  (let-thread-fluid $umask new-umask thunk))
-
-(define (with-umask* new-umask thunk)
-  (let ((changed-umask
-	 (with-lock umask-lock
-	   (lambda ()
-	     (change-and-cache-umask new-umask)
-	     (umask-cache)))))
-    (let-umask changed-umask thunk)))
-
-(define (align-umask!)
-  (let ((thread-umask (umask)))
-    (if (not (= thread-umask (umask-cache)))
-	(change-and-cache-umask thread-umask))))
-
-(define (set-umask new-umask)
-  (with-lock umask-lock
-    (lambda ()
-      (change-and-cache-umask new-umask)
-      (thread-set-umask! (umask-cache)))))
-
-(define umask-resource (make-resource align-umask! umask-lock))
-
-(define umask-reinitializer
-  (make-reinitializer (lambda () (initialize-umask))))
 
+(make-process-resource 
+ initialize-umask umask thread-set-umask set-umask 
+ with-umask* with-umask-aligned*
+ process-umask set-process-umask =)
 
 (initialize-umask)
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; effective uid and gid per thread
 
-(define-syntax make-Xid-resource 
-  (syntax-rules ()
-    ((make-Xid-resource 
-      process-user-effective-Xid set-process-user-effective-Xid
-      process-set-Xid set-Xid
-      align-eXid! eXid-resource
-      user-effective-Xid set-user-effective-Xid with-user-effective-Xid*)
-     (begin
-       (define *eXid-cache* 'uninitialized)
-       (define eXid-lock (make-lock))
-     
-       (define (initialize-eXid)
-	 (set! *eXid-cache* (process-user-effective-Xid))
-	 (set! $eXid
-	       (make-preserved-thread-fluid
-		(eXid-cache))))
-
-       (define (eXid-cache)
-	 *eXid-cache*)
-
-;; Actually do the syscall and update the cache
-;; assumes the resource lock obtained
-       (define (change-and-cache-eXid new-eXid)
-	 (set-process-user-effective-Xid new-eXid)
-	 (set! *eXid-cache* (process-user-effective-Xid)))
-
-;; The thread-specific eXid: A thread fluid
-
-       (define $eXid 'empty-eXid-value)
-
-       (define (user-effective-Xid) (thread-fluid $eXid))
-       (define (thread-set-eXid! new-eXid) (set-thread-fluid! $eXid new-eXid))
-       (define (let-eXid new-eXid thunk)
-	 (let-thread-fluid $eXid new-eXid thunk))
-
-;; set-Xid will affect the effective X id
-       (define (set-Xid Xid)
-	 (with-lock eXid-lock
-	   (lambda ()
-	     (process-set-Xid Xid)
-	     (set! *eXid-cache* (process-user-effective-Xid))
-	     (thread-set-eXid! *eXid-cache*))))
-
-       (define (with-user-effective-Xid* new-eXid thunk)
-	 (let ((changed-eXid
-		(with-lock eXid-lock
-		  (lambda ()
-		    (change-and-cache-eXid new-eXid)
-		    (eXid-cache)))))
-	   (let-eXid changed-eXid thunk)))
-
-       (define (align-eXid!)
-	 (let ((thread-eXid (user-effective-Xid)))
-	   (if (not (= thread-eXid (eXid-cache)))
-	       (change-and-cache-eXid thread-eXid))))
-
-       (define (set-user-effective-Xid new-eXid)
-	 (with-lock eXid-lock
-	   (lambda ()
-	     (change-and-cache-eXid new-eXid)
-	     (thread-set-eXid! (eXid-cache)))))
-
-       (define eXid-resource (make-resource align-eXid! eXid-lock))
-
-       (define eXid-reinitializer
-	 (make-reinitializer (lambda () (initialize-eXid))))
-
-       (initialize-eXid)
-       ))))
-
-(make-Xid-resource 
- process-user-effective-uid set-process-user-effective-uid
- process-set-uid set-uid
- align-euid! euid-resource
- user-effective-uid set-user-effective-uid with-user-effective-uid*)
-
-(make-Xid-resource 
- process-user-effective-gid set-process-user-effective-gid
- process-set-gid set-gid
- align-egid! egid-resource
- user-effective-gid set-user-effective-gid with-user-effective-gid*)
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; ensure S48 is aligned too
-
-(set-with-fs-context-aligned*! 
+(set-with-fs-context-aligned*! ; ensure S48 is aligned too
  (lambda (thunk)
-   (with-resources-aligned 
-    (list cwd-resource umask-resource euid-resource egid-resource) 
-    thunk)))
+   (with-cwd-aligned*
+    (lambda ()
+      (with-umask-aligned*
+       thunk)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Environment per thread
-(define *env-cache* 'uninitialized)
-(define env-lock (make-lock))
-
-(define (install-env)
-  (set! *env-cache* (environ**-read))
-  (set! $env ;;; TODO The old thread-fluid will remain
-	(make-preserved-thread-fluid
-	 (env-cache))))
-;  (set! env-lock (make-lock)))
-
-(define (env-cache)
-  *env-cache*)
-
-;; Actually do the syscall and update the cache
-;; assumes the env lock obtained
-(define (change-and-cache-env env)
-  (environ**-set env)
-  (set! *env-cache* env))
-
-;; The thread-specific env: A thread fluid
-
-(define $env 'empty-env-value)
-
-(define (thread-read-env) (thread-fluid $env))
-(define (thread-set-env! res) (set-thread-fluid! $env res))
-(define (let-env res thunk)
-  (let-thread-fluid $env res thunk))
-
-(define (really-with-env* env thunk)
-  (with-lock env-lock
-    (lambda ()
-      (change-and-cache-env env)))
-  (let-env env thunk))
-
-(define (align-env!)
-  (let ((res (thread-read-env)))
-    (if (not (env=? res (env-cache)))
-	(change-and-cache-env res))))
-
-(define (thread-change-env res)
-  (with-lock env-lock
-    (lambda ()
-      (change-and-cache-env res)
-      (thread-set-env! (env-cache)))))
-
-(define environ-resource (make-resource align-env! env-lock))
-
-(define env-reinitializer
-  (make-reinitializer install-env))
-
 
 (define-record env
       envvec
@@ -474,18 +303,15 @@
 
 (define (setenv var val)
   (let* ((env (thread-read-env))
-	 (alist (if val
-		    (alist-update 
-		     var 
-		     val 
-		    (env:alist env))
-		    (alist-delete
-		     var
-		     (env:alist env)))))
+	 (alist (alist-update 
+		 var 
+		 val 
+		 (fold cons '() (env:alist env)))))
     (thread-set-env! 
      (make-env 
       #f
-      alist))))
+      alist
+      ))))
 
 (define (alist->env alist)
   (thread-set-env!
@@ -501,8 +327,12 @@
     (with-total-env* new-env thunk)))
 
 (define (with-total-env* alist thunk)
-  (really-with-env* (make-env #f alist) thunk))
+  (with-env-internal* (make-env #f alist) thunk))
 
+(make-process-resource install-env thread-read-env thread-set-env! 
+		       useless-set-env
+		       with-env-internal* with-env-aligned* 
+		       environ**-read environ**-set env=?)
 
 ;;; These two functions are obsoleted by the more general INFIX-SPLITTER and
 ;;; JOIN-STRINGS functions. However, we keep SPLIT-COLON-LIST defined
@@ -587,6 +417,12 @@
 (define-simple-syntax (with-cwd dir . body)
   (with-cwd* dir (lambda () . body)))
 
+(define-simple-syntax (with-cwd-aligned body ...)
+  (with-cwd-aligned* (lambda () body ...)))
+
+(define-simple-syntax (with-umask-aligned body ...)
+  (with-cwd-aligned* (lambda () body ...)))
+
 (define-simple-syntax (with-umask mask . body)
   (with-umask* mask (lambda () . body)))
 
@@ -596,11 +432,6 @@
 (define-simple-syntax (with-total-env env . body)
   (with-total-env* `env (lambda () . body)))
 
-(define-simple-syntax (with-user-effective-uid uid . body)
-  (with-user-effective-uid* uid (lambda () . body)))
-
-(define-simple-syntax (with-user-effective-gid gid . body)
-  (with-user-effective-gid* gid (lambda () . body)))
 
 (define (call/temp-file writer user)
   (let ((fname #f))
@@ -655,7 +486,7 @@
 	  (let ((fname (format #f template (number->string i))))
 	    (receive retvals (with-errno-handler
 			       ((errno data)
-				((errno/exist errno/acces) #f))
+				((errno/exist) #f))
 			       (maker fname))
 	      (if (car retvals) (apply values retvals)
 		  (loop (+ i 1)))))))))
@@ -896,7 +727,7 @@
 ;; Loop until EOF reading characters or strings and writing (FILTER char)
 ;; or (FILTER string). Useful as an arg to FORK or FORK/PIPE.
 
-(define (make-char-port-filter filter)
+(define (char-filter filter)
   (lambda ()
     (let lp ()
       (let ((c (read-char)))
@@ -904,12 +735,12 @@
 	    (begin (write-char (filter c))
 		   (lp)))))))
 
-(define (make-string-port-filter filter . maybe-buflen)
+(define (string-filter filter . maybe-buflen)
   (let* ((buflen (:optional maybe-buflen 1024))
 	 (buf (make-string buflen)))
     (lambda ()
       (let lp ()
-	(cond ((read-string! buf (current-input-port) 0 buflen) =>
+	(cond ((read-string! buf 0 buflen) =>
 	       (lambda (nread)
 		 (display (filter (if (= nread buflen) buf
 				      (substring buf 0 nread)))) ; last one.
@@ -1014,25 +845,22 @@
 		     thing))))
 
 (define (exec-path-search prog path-list)
-  (cond
-   ((not (file-name-absolute? prog))
-    (let loop ((path-list path-list))
-      (if (not (null? path-list))
-	  (let* ((dir (car path-list))
-		 (fname (string-append dir "/" prog)))
-	    (if (file-executable? fname)
-		fname
-		(loop (cdr path-list)))))))
-   ((file-executable? prog)
-    prog)
-   (else #f)))
-
+  (if (file-name-absolute? prog)
+      (and (file-executable? prog) prog)
+      (first? (lambda (dir)
+		(let ((fname (string-append dir "/" prog)))
+		  (and (file-executable? fname) fname)))
+	     path-list)))
+		    
 (define (exec/env prog env . arglist)
   (flush-all-ports)
-  (with-resources-aligned 
-   (list environ-resource cwd-resource umask-resource euid-resource egid-resource)
+  (with-env-aligned* 
    (lambda ()
-     (%exec prog (cons prog arglist) env))))
+     (with-cwd-aligned*
+      (lambda ()
+	(with-umask-aligned*
+	 (lambda ()
+	   (%exec prog (cons prog arglist) env))))))))
 
 ;(define (exec-path/env prog env . arglist)
 ;  (cond ((exec-path-search (stringify prog) (fluid exec-path-list)) =>
@@ -1046,23 +874,26 @@
 
 (define (exec-path/env prog env . arglist)
   (flush-all-ports)
-  (with-resources-aligned 
-   (list environ-resource cwd-resource umask-resource euid-resource egid-resource)
+  (with-env-aligned*
    (lambda ()
-     (let ((prog (stringify prog)))
-       (if (string-index prog #\/)
+     (with-cwd-aligned*
+      (lambda ()
+	(with-umask-aligned*
+	 (lambda ()
+	   (let ((prog (stringify prog)))
+	     (if (string-index prog #\/)
 		 
-	   ;; Contains a slash -- no path search.
-	   (%exec prog (cons prog arglist) env)
+		 ;; Contains a slash -- no path search.
+		 (%exec prog (cons prog arglist) env)
 		 
-	   ;; Try each directory in PATH-LIST.
-	   (let ((argv (list->vector (cons prog (map stringify arglist)))))
-	     (for-each (lambda (dir)
-			 (let ((binary (string-append dir "/" prog)))
-			   (%%exec binary argv env)))
-		       (thread-fluid exec-path-list)))))
+		 ;; Try each directory in PATH-LIST.
+		 (let ((argv (list->vector (cons prog (map stringify arglist)))))
+		   (for-each (lambda (dir)
+			       (let ((binary (string-append dir "/" prog)))
+				 (%%exec binary argv env)))
+			     (fluid exec-path-list)))))
 	   
-     (error "No executable found." prog arglist))))
+	   (error "No executable found." prog arglist))))))))
    
 (define (exec-path prog . arglist)
   (apply exec-path/env prog #t arglist))
@@ -1073,97 +904,42 @@
 
 ;;; Assumes niladic primitive %%FORK.
 
-(define (fork . stuff)
-  (apply fork-1 #t stuff))
+(define (fork . maybe-thunk)
+  (flush-all-ports)
+  (really-fork #t maybe-thunk))
 
-(define (%fork . stuff)
-  (apply fork-1 #f stuff))
+(define (%fork . maybe-thunk)
+  (really-fork #f maybe-thunk))
 
-(define (fork-1 clear-interactive? . rest)
-  (let-optionals rest ((maybe-thunk #f)
-		       (dont-narrow? #f))
-    (really-fork clear-interactive?
-		 (not dont-narrow?)
-		 maybe-thunk)))
 
-(define (preserve-ports thunk)
-  (let ((current-input (current-input-port))
-	(current-output (current-output-port))
-	(current-error (current-error-port)))
-    (lambda ()
-      (with-current-input-port*
-       current-input
-       (lambda ()
-	 (with-current-output-port*
-	  current-output
-	  (lambda ()
-	    (with-current-error-port*
-	     current-error
-	     thunk))))))))
+(define (really-fork clear-interactive? maybe-thunk)
+  (with-env-aligned* ; not neccessary here but doing it on exec
+                     ; genereates no cache in the parent
+   (lambda ()
+   (((structure-ref interrupts with-interrupts-inhibited) (lambda ()
+     (let ((pid (%%fork)))
+       (if (zero? pid)				
 
-(define (really-fork clear-interactive? narrow? maybe-thunk)
-  (let ((proc #f)
-	(maybe-narrow
-	 (if narrow?
-	     (lambda (thunk)
-	       ;; narrow loses the thread fluids and the dynamic environment
-	       (narrow (preserve-ports (preserve-thread-fluids thunk))
-		       'forking))
-	     (lambda (thunk) (thunk)))))
-    (maybe-narrow
-     (lambda ()
+	   ;; Child
+	   (lambda ()	; Do all this outside the WITH-INTERRUPTS.
+;	     (set! reaped-procs '())
 
-       (if clear-interactive?
-	   (flush-all-ports))
+	     ;;; There is no session if parent was started in batch-mode
+	     (if (and (session-started?) clear-interactive?)
+		 (set-batch-mode?! #t))	; Children are non-interactive.
+	     (and (pair? maybe-thunk)
+		  (call-terminally (car maybe-thunk))))
 
-       ;; There was an atomicity problem/race condition -- if a child
-       ;; process died after it was forked, but before the scsh fork
-       ;; procedure could register the child's procobj in the
-       ;; pid/procobj table, then when the SIGCHLD signal-handler reaped
-       ;; the process, there would be no procobj for it.  We now lock
-       ;; out interrupts across the %%FORK and NEW-CHILD-PROC
-       ;; operations.
+	   ;; Parent
+	   (let ((proc (new-child-proc pid)))
+	     (lambda () proc))))))))))
 
-       (((structure-ref interrupts with-interrupts-inhibited)
-	 (lambda ()
-	   ;; with-env-aligned is not neccessary here but it will
-	   ;; create the environ object in the parent process which
-	   ;; could reuse it on further forks
-	   (let ((pid (with-resources-aligned 
-		       (list environ-resource)
-		       %%fork)))
-	     (if (zero? pid)				
-		 ;; Child
-		 (lambda ()		; Do all this outside the WITH-INTERRUPTS.
-                   (if narrow?
-                       (begin
-                         ;; ugly kludge:
-                         ;; the REPL thread is not running any more,
-                         ;; hence unlock its command ports
-                         (release-port-lock (command-input))
-                         (release-port-lock (command-output))
-                         (release-port-lock (command-error-output))))
-		   ;; There is no session if parent was started in batch-mode
-		   (if (and (session-started?) clear-interactive?)
-		       (set-batch-mode?! #t)) ; Children are non-interactive.
-		   (if maybe-thunk
-		       (call-terminally maybe-thunk)))
-		 ;; Parent
-		 (begin
-		   (set! proc (new-child-proc pid))
-		   (lambda ()
-		     (values))))))))))
-    proc))
 
 (define (exit . maybe-status)
-  (let ((status (:optional  maybe-status 0)))
-    (if (not (integer? status))
-        (error "non-integer argument to exit"))
-    (call-exit-hooks-and-narrow
-     (lambda ()
-       (exit/errno status)
-       (display "The evil undead walk the earth." 2)
-       (if #t (error "(exit) returned."))))))
+  (flush-all-ports)
+  (exit/errno (:optional  maybe-status 0))
+  (display "The evil undead walk the earth." 2)
+  (if #t (error "(exit) returned.")))
 
 
 ;;; The classic T 2.0 primitive.
@@ -1183,7 +959,7 @@
 
 ;;; Some globals:
 (define home-directory "")
-(define exec-path-list)
+(define exec-path-list (make-fluid '()))
 
 (define (init-scsh-vars quietly?)
   (set! home-directory
@@ -1201,12 +977,11 @@
 			(lambda ()
 			  (user-info:home-dir (user-info (user-uid))))))))))
 	      
-  (set! exec-path-list
-	(make-preserved-thread-fluid 
-	 (cond ((getenv "PATH") => split-colon-list)
-	       (else (if (not quietly?)
-			 (warn "Starting up with no path ($PATH)."))
-		     '())))))
+  (set-fluid! exec-path-list
+	      (cond ((getenv "PATH") => split-colon-list)
+		    (else (if (not quietly?)
+			      (warn "Starting up with no path ($PATH)."))
+			  '()))))
 
 
 ; SIGTSTP blows s48 away. ???
