@@ -1,5 +1,5 @@
 ;;; Scsh start-up code.
-;;; Copyright (c) 1995 by Olin Shivers. See file COPYING.
+;;; Copyright (c) 1995 by Olin Shivers.
 
 ;;; A scsh starter takes the command line args, parses them, 
 ;;; initialises the scsh system, and either starts up a repl loop
@@ -25,10 +25,11 @@
   (let ((filename (translate filename)))
     (display (string-append "Writing " filename) (command-output))
     (newline (command-output))
-    (flush-the-symbol-table!)	;Gets restored at next use of string->symbol
+    ;JMG: it is set to #f in the vm, so I omit it now
+;;;(flush-the-symbol-table!)	;Gets restored at next use of string->symbol
     (write-image filename
 		 (scsh-stand-alone-resumer start)
-		 "")
+		 "Scsh 0.6")
     #t))
 
 
@@ -42,23 +43,42 @@
 ;;; This is what we export to the user for his programs.
 
 (define (dump-scsh-program start filename)
+  (let ((context (user-context)))
   (really-dump-scsh-program (lambda (args)
-			      (init-scsh-vars #t)	; Do it quietly.
-			      (start args))
-			    filename))
+			      (with-scsh-sighandlers 
+			       #f
+			       (lambda ()
+				 (with-autoreaping
+				  (lambda ()
+				    (install-env)
+				    (initialize-cwd)
+				    (init-scsh-vars #f)	; Do it quietly.
+				    (start-new-session context    
+						       (current-input-port)
+						       (current-output-port)
+						       (current-error-port)
+						       args
+						       #f)
+				    (with-interaction-environment     
+				     (user-environment)
+				     (lambda ()
+				       (start args))))))))
+				  filename)))
 
 
 (define (scsh-stand-alone-resumer start)
   (usual-resumer  ;sets up exceptions, interrupts, and current input & output
    (lambda (args)	; VM gives us our args, but not our program.
-     (init-scsh-hindbrain #t)	; Whatever. Relink & install scsh's I/O system.
+     (init-fdports!)
      (call-with-current-continuation
        (lambda (halt)
 	 (set! %vm-prog-args (cons "scsh" args))	; WRONG -- use image.
 	 (set-command-line-args! %vm-prog-args)
-	 (with-handler (simple-condition-handler halt (error-output-port))
-	   (lambda ()
-	     (let ((exit-val (start (command-line))))
-	       (if (integer? exit-val) exit-val 0))))))))) ; work around bug.
+	 (with-handler 
+	  (simple-condition-handler halt (current-error-port))
+	  (lambda ()
+	    (let ((exit-val (start (command-line))))
+	      (if (integer? exit-val) exit-val 0))))))))) ; work around bug.
 
 (define %vm-prog-args #f)
+ 
\ No newline at end of file
