@@ -1,5 +1,5 @@
-#!/usr/local/lib/scsh/scshvm \
--o /usr/local/lib/scsh/scshvm -h 8000000 -i /usr/local/lib/scsh/scsh.image -lm /usr/local/lib/scsh/vm/ps-interface.scm -lm /usr/local/lib/scsh/vm/interfaces.scm -lm /usr/local/lib/scsh/vm/package-defs.scm -lm /usr/local/lib/scsh/vm/s48-package-defs.scm -dm -m static-heaps -e static-heap-linker -s
+#!/afs/informatik.uni-tuebingen.de/i386_fbsd43/scsh-0.6.1//lib/scsh/scshvm \
+-o /afs/informatik.uni-tuebingen.de/i386_fbsd43/scsh-0.6.1//lib/scsh/scshvm -h 8000000 -i /afs/informatik.uni-tuebingen.de/i386_fbsd43/scsh-0.6.1//lib/scsh/scsh.image -lm /afs/informatik.uni-tuebingen.de/i386_fbsd43/scsh-0.6.1//lib/scsh/vm/ps-interface.scm -lm /afs/informatik.uni-tuebingen.de/i386_fbsd43/scsh-0.6.1//lib/scsh/vm/interfaces.scm -lm /afs/informatik.uni-tuebingen.de/i386_fbsd43/scsh-0.6.1//lib/scsh/vm/package-defs.scm -lm /afs/informatik.uni-tuebingen.de/i386_fbsd43/scsh-0.6.1//lib/scsh/vm/s48-package-defs.scm -dm -m static-heaps -e static-heap-linker -s
 !#
 
 #!
@@ -128,7 +128,7 @@ For testing load this at a scsh prompt
 	 options
 	 (or (options:temp-dir options)
 	     (getenv "TMPDIR") 
-	     "/usr/tmp"))
+	     "/var/tmp"))
 	(set-options:cc-command
 	 options
 	 (or (options:cc-command options)
@@ -452,7 +452,7 @@ For testing load this at a scsh prompt
 					 (cons "static.o"
 					       l)))))
 			       (options:args-parser options)
-			       '("-L" "/usr/local/lib/scsh" "-lscshvm")
+			       '("-L" "/afs/informatik.uni-tuebingen.de/i386_fbsd43/scsh-0.6.1//lib/scsh" "-lscshvm")
 			       libs)))
 	  (message command)
 	  (run (,@command)))))
