@@ -46,7 +46,7 @@
 #include "cstuff.h"
 #include "time1.h"	/* Make sure the .h interface agrees with the code. */
 
-#include <stdio.h>     // JMG debug
+#include <stdio.h>     /* JMG debug */
 
 extern char **environ;
 
@@ -76,12 +76,12 @@ static char **make_newenv(s48_value zone, char *newenv[2])
     
     char * extracted_zone = s48_extract_string(zone);
     
-    // s48_value temp;
+    /* s48_value temp; */
     if( !tz ) return NULL;
     strcpy(tz, "TZ=");
 
-    //    temp = S48_STRING_REF(zone,0); //JMG
-    // strncpy(tz+3, &temp, zonelen);
+    /*    temp = S48_STRING_REF(zone,0);*/ /* JMG */
+    /* strncpy(tz+3, &temp, zonelen);*/
     strncpy(tz+3, extracted_zone, zonelen);
     tz[zonelen+3] = '\0';
     newenv[0] = tz;
@@ -113,19 +113,6 @@ s48_value scheme_time()
     return s48_enter_integer (t);
 }
 
-// should be part of the FFI interface
-s48_value s48_list_11 (s48_value e1, s48_value e2, s48_value e3, 
-		       s48_value e4, s48_value e5, s48_value e6, 
-		       s48_value e7, s48_value e8, s48_value e9, 
-		       s48_value e10, s48_value e11)
-{
-  return 
-    s48_cons (e1, s48_cons (e2, s48_cons (e3, s48_cons (e4, s48_cons 
-	(e5, s48_cons (e6, s48_cons (e7, s48_cons (e8, s48_cons (e9, 
-     s48_cons (e10, s48_cons (e11, S48_NULL)))))))))));
-}
-
-
 /* Zone:
 **   #f		Local time
 **   int	Offset from GMT in seconds.
@@ -255,7 +242,7 @@ s48_value date2time(s48_value sec, s48_value min, s48_value hour,
 	t = mktime(&d);
 	/* t == -1 => you probably have an error. */
 	if ((t == -1) && (errno != 0))
-	  // Sorry, we only have a version with 5 arguments...
+	  /* Sorry, we only have a version with 5 arguments..*/
 	  s48_raise_os_error_5 (errno, sec, min, hour, mday, month);
 	t -= s48_extract_fixnum(tz_secs);
 	environ = oldenv;
