@@ -344,7 +344,10 @@
 		    args 
 		    context 
 		    (lambda () 
-		      (display "Welcome to scsh 0.6.0 (Chinese Democracy)" 
+		      (display (string-append 
+				"Welcome to scsh "
+				scsh-version-string
+				" (Combinatorial Algorithms)")
 			       (current-output-port))
 		      (newline (current-output-port))
 		      (display "Type ,? for help."
@@ -356,13 +359,13 @@
 		  (let ((result (eval (read-exactly-one-sexp-from-string term-val)
 				      (interaction-environment))))
 		    (call-exit-hooks)
-		    (scheme-exit-now result)))
+		    (scheme-exit-now 0)))
 		 
 		 (top-entry		; There was a -e <entry>.
 		  (let ((result ((eval top-entry (interaction-environment))
 				    (command-line))))
 		    (call-exit-hooks)
-		    (scheme-exit-now result)))
+		    (scheme-exit-now 0)))
 		 
 		     ;; Otherwise, the script executed as it loaded,
 		 ;; so we're done.
@@ -378,7 +381,9 @@
 
 ;;; placeholder for an extensible mechanism in the future
 (define (call-exit-hooks)
-  (flush-all-ports))
+  (flush-all-ports)
+  (relinquish-timeslice)
+  (relinquish-timeslice))
 
 (define (load-library-file file lib-dirs script-file)
 ; (format (error-output-port) "Load-library-file: ~a ~s\n" file lib-dirs)
